import 'pagination_utils.dart';
import 'package:flutter/material.dart';
import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:equatable/equatable.dart';

final paginationControllerProvider = ChangeNotifierProvider.autoDispose
    .family<PaginationController, ControllerParameters>((ref, parameters) {
  final paginationController =
      PaginationController.initial(parameters: parameters);
  return paginationController;
});

class ControllerParameters extends Equatable {
  const ControllerParameters({this.service, this.resultsPerPage});

  final PaginationService service;
  final int resultsPerPage;

  @override
  List<Object> get props => [service, resultsPerPage];
}

class PaginationController extends ChangeNotifier {
  List itemList = [];
  int pageIndex;
  String errorMessage;
  dynamic lastFetched;
  PaginationService service;
  int resultsPerPage;

  PaginationController.initial({@required ControllerParameters parameters}) {
    service = parameters.service;
    resultsPerPage = parameters.resultsPerPage;

    itemList = [];
    pageIndex = 0;
    errorMessage = '';
    lastFetched = 0;
    fetch();
  }

  ///Set [pageIndex]
  setPageIndex(int p) {
    pageIndex = p;
    notifyListeners();
  }

  ///Increase [pageIndex]
  increasePageIndex() async {
    if (itemList.length >=
        resultsPerPage) //If there are more pages according to the list size
    {
      if (itemList.length ~/ resultsPerPage - 1 ==
          pageIndex) // When pageIndex is the second-last fetch!
      {
        print('anticipated fetch');
        await fetch();
      }
      if (pageIndex * resultsPerPage + resultsPerPage < getSize) {
        pageIndex++;
//        print('pageIndex now: $pageIndex');
        notifyListeners();
      }
//      else
//        print('end of list');
    } else {
      fetch();
      if (itemList.length >= resultsPerPage) {
        if (pageIndex * resultsPerPage + resultsPerPage < getSize) {
          pageIndex++;
          notifyListeners();
        }
//        else
//          print('end of list');
      }
    }
  }

  ///Decrease [pageIndex]
  decreasePageIndex() {
    if (pageIndex > 0) pageIndex--;
    notifyListeners();
  }

  /// Fetches more data from [service]
  Future<void> fetch() async {
    try {
      final newItems = await service.getData(lastFetched);
      if (newItems.length > 0) {
        lastFetched = newItems[newItems.length - 1];
        itemList.addAll(newItems);
        notifyListeners();
      }
    } catch (e) {
      print(e.toString());
      if ("NoSuchMethodError: invalid member on null: 'length'" == e.toString())
        errorMessage = 'Empty List';
      else
        errorMessage = e.toString();
    }
    print('size: ${itemList.length}');
  }

  /// The items that are actually displayed in the page
  List displayItems() {
    final itemsDisplayed = itemList.sublist(
        (pageIndex * resultsPerPage),
        (pageIndex + 1) * resultsPerPage >= itemList.length
            ? itemList.length
            : pageIndex * resultsPerPage + resultsPerPage);
    return itemsDisplayed;
  }

  bool get refreshError => errorMessage != '';
  get getList => itemList;
  get getSize => itemList.length;
}
